;;; DO NOT MODIFY THIS FILE
(if (featurep 'mew-autoloads) (error "Already loaded"))

(provide 'mew-autoloads)

;;;### (autoloads nil "_pkg" "mew/_pkg.el")

(package-provide 'mew :version 1.09 :type 'regular)

;;;***
